@@ -35,6 +35,8 @@ from gajim.common.events import PlainConnection
 from gajim.common.events import SignedIn
 from gajim.common.events import StanzaReceived
 from gajim.common.events import StanzaSent
+from gajim.common.file_transfer_manager import FileTransfer
+from gajim.common.helpers import determine_proxy
 from gajim.common.helpers import get_account_proxy
 from gajim.common.helpers import get_custom_host
 from gajim.common.helpers import get_resource
@@ -46,7 +48,6 @@ from gajim.common.idle import Monitor
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.message import build_message_stanza
 from gajim.common.structs import OutgoingMessage
-from gajim.common.util.http import create_http_session
 from gajim.common.util.standards import get_rfc5646_lang
 from gajim.common.util.status import get_idle_status_message
 from gajim.common.util.text import to_one_line
@@ -170,7 +171,6 @@ class Client(Observable, ClientModules):
         self._client.set_domain(self._address.domain)
         self._client.set_username(self._address.localpart)
         self._client.set_resource(get_resource(self._account))
-        self._client.set_http_session(create_http_session())
         self._client.set_supported_fallback_ns([Namespace.REPLY])
 
         self._client.subscribe('resume-failed', self._on_resume_failed)
@@ -311,9 +311,12 @@ class Client(Observable, ClientModules):
                              text=text or error))
 
         if self._reconnect:
+            # Save resumeable information because in
+            # _after_disconnect() we destroy the client object
+            stream_is_resumeable = self._client.resumeable
             self._after_disconnect()
             self._schedule_reconnect()
-            if not self._client.resumeable:
+            if not stream_is_resumeable:
                 self.notify('state-changed', SimpleClientState.DISCONNECTED)
             self.notify('state-changed', SimpleClientState.RESUME_IN_PROGRESS)
 
@@ -482,6 +485,7 @@ class Client(Observable, ClientModules):
         self.get_module('LastActivity').set_enabled(True)
         self.get_module('Annotations').request_annotations()
         self.get_module('Blocking').get_blocking_list()
+        self.get_module('VCard4').subscribe_to_node()
 
         if app.settings.get_account_setting(self._account, 'publish_tune'):
             self.get_module('UserTune').set_enabled(True)
@@ -554,6 +558,7 @@ class Client(Observable, ClientModules):
             # Do not try to reco while we are already trying
             return
 
+        assert self._client is not None
         custom_host = get_custom_host(self._account)
         if custom_host is not None:
             self._client.set_custom_host(*custom_host)
@@ -595,7 +600,28 @@ class Client(Observable, ClientModules):
                                                 abort=self._abort_reconnect))
             return
 
-        self._client.connect()
+        def _on_host_meta_response(obj: FileTransfer) -> None:
+            if self._client is None or self._state != ClientState.CONNECTING:
+                # State was changed in the meantime
+                return
+
+            try:
+                result = obj.get_result()
+            except Exception as error:
+                log.warning('Error while requesting host-meta data: %s', error)
+            else:
+                log.info("Received host meta data with length: %s", len(result.content))
+                self._client.set_host_meta_data(result.content)
+
+            self._client.connect()
+
+        app.ftm.http_request(
+            'GET',
+            f'https://{self._address.domain}/.well-known/host-meta',
+            proxy=determine_proxy(self._account),
+            timeout=3,
+            callback=_on_host_meta_response,
+        )
 
     def _schedule_reconnect(self) -> None:
         self._set_state(ClientState.RECONNECT_SCHEDULED)
