@@ -28,7 +28,6 @@ HAS_ACCOUNT_DEFAULT = _ACCOUNTDEFAULT()
 BoolSettings = Literal[
     'always_english_wikipedia',
     'always_english_wiktionary',
-    'ascii_formatting',
     'ask_online_status',
     'autoaway',
     'autoxa',
@@ -37,11 +36,8 @@ BoolSettings = Literal[
     'confirm_close_muc',
     'confirm_on_window_delete',
     'dev_force_bookmark_2',
-    'developer_modus',
     'enable_emoji_shortcodes',
     'enable_keepassxc_integration',
-    'enable_negative_priority',
-    'enable_file_preview',
     'escape_key_closes',
     'gc_notify_on_all_messages_private_default',
     'gc_notify_on_all_messages_public_default',
@@ -64,13 +60,11 @@ BoolSettings = Literal[
     'preview_verify_https',
     'print_status_in_chats',
     'remote_control',
-    'show_main_menu',
     'send_on_ctrl_enter',
     'show_help_start_chat',
     'show_in_taskbar',
     'show_notifications',
     'show_notifications_away',
-    'show_only_chat_and_online',
     'show_send_message_button',
     'show_voice_message_button',
     'show_status_msgs_in_roster',
@@ -87,6 +81,7 @@ BoolSettings = Literal[
     'use_stun_server',
     'use_urgency_hint',
     'video_see_self',
+    'show_header_bar',
 ]
 
 IntSettings = Literal[
@@ -132,8 +127,8 @@ StringSettings = Literal[
     'muclumbus_api_http_uri',
     'muclumbus_api_jid',
     'muclumbus_api_pref',
-    'preview_leftclick_action',
     'providers_list_url',
+    'quick_reaction_emojis',
     'roster_theme',
     'search_engine',
     'show_main_window_on_startup',
@@ -153,7 +148,6 @@ APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'always_english_wikipedia': False,
     'always_english_wiktionary': True,
     'app_font_size': 1.0,
-    'ascii_formatting': True,
     'ask_online_status': False,
     'audio_input_device': 'autoaudiosrc ! volume name=gajim_vol',
     'audio_input_volume': 50,
@@ -175,12 +169,9 @@ APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'date_format': '%x',
     'date_time_format': '%c',
     'dev_force_bookmark_2': False,
-    'developer_modus': False,
     'dictionary_url': 'WIKTIONARY',
     'enable_emoji_shortcodes': True,
     'enable_keepassxc_integration': False,
-    'enable_negative_priority': False,
-    'enable_file_preview': True,
     'escape_key_closes': False,
     'file_transfers_port': 28011,
     'ft_add_hosts_to_send': '',
@@ -219,23 +210,22 @@ APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'positive_184_ack': False,
     'preview_allow_all_images': False,
     'preview_anonymous_muc': False,
-    'preview_leftclick_action': 'open',
     'preview_max_file_size': 10485760,
     'preview_size': 400,
     'preview_verify_https': True,
     'providers_list_url': 'https://data.xmpp.net/providers/v2/providers-B.json',
     'print_status_in_chats': False,
+    'quick_reaction_emojis': '👍,❤,🤣',
     'remote_control': False,
     'roster_theme': 'default',
     'search_engine': 'https://duckduckgo.com/?q=%s',
     'send_on_ctrl_enter': False,
+    'show_header_bar': True,
     'show_help_start_chat': True,
     'show_in_taskbar': True,
-    'show_main_menu': True,
     'show_main_window_on_startup': 'always',
     'show_notifications': True,
     'show_notifications_away': False,
-    'show_only_chat_and_online': False,
     'show_send_message_button': False,
     'show_voice_message_button': True,
     'show_status_msgs_in_roster': True,
@@ -263,7 +253,6 @@ APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
 
 BoolAccountSettings = Literal[
     'active',
-    'adjust_priority_with_status',
     'anonymous_auth',
     'answer_receipts',
     'autoauth',
@@ -285,7 +274,6 @@ BoolAccountSettings = Literal[
     'send_idle_time',
     'send_marker_default',
     'send_os_info',
-    'send_time_info',
     'sync_muc_blocks',
     'sync_with_global_status',
     'test_ft_proxies_on_startup',
@@ -323,14 +311,8 @@ StringAccountSettings = Literal[
 ]
 
 IntAccountSettings = Literal[
-    'autopriority_away',
-    'autopriority_chat',
-    'autopriority_dnd',
-    'autopriority_online',
-    'autopriority_xa',
     'chat_history_max_age',
     'custom_port',
-    'priority',
 ]
 
 
@@ -389,7 +371,6 @@ ACCOUNT_SETTINGS = {
         'account_color': 'rgb(85, 85, 85)',
         'account_label': '',
         'active': False,
-        'adjust_priority_with_status': False,
         'address': '',
         'anonymous_auth': False,
         'answer_receipts': True,
@@ -397,11 +378,6 @@ ACCOUNT_SETTINGS = {
         'autoauth': False,
         'autoconnect': True,
         'autojoin_sync': True,
-        'autopriority_away': 40,
-        'autopriority_chat': 50,
-        'autopriority_dnd': 20,
-        'autopriority_online': 50,
-        'autopriority_xa': 30,
         'avatar_sha': '',
         'chat_history_max_age': -1,
         'client_cert': '',
@@ -427,7 +403,6 @@ ACCOUNT_SETTINGS = {
         'last_status': 'online',
         'last_status_msg': '',
         'password': '',
-        'priority': 0,
         'proxy': '',
         'publish_location': False,
         'publish_tune': False,
@@ -440,7 +415,6 @@ ACCOUNT_SETTINGS = {
         'send_idle_time': True,
         'send_marker_default': True,
         'send_os_info': True,
-        'send_time_info': True,
         'subscription_request_msg': '',
         'sync_muc_blocks': True,
         'sync_with_global_status': True,
@@ -616,8 +590,6 @@ ADVANCED_SETTINGS = {
             'IANA-registered ones. Space-separated, lower-case, no colons.'),
         'always_english_wikipedia': '',
         'always_english_wiktionary': '',
-        'ascii_formatting': _(
-            'Treat * / _ pairs as possible formatting characters.'),
         'chat_merge_consecutive_nickname': _(
             'Show message meta data (avatar, nickname, timestamp) only once, '
             'if there are multiple messages from the same sender within a '
@@ -631,15 +603,9 @@ ADVANCED_SETTINGS = {
         'date_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
         'date_time_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
         'dev_force_bookmark_2': _('Force Bookmark 2 usage'),
-        'developer_modus': '',
         'dictionary_url': _(
             'Either a custom URL with %%s in it (where %%s is the word/phrase)'
             ' or "WIKTIONARY" (which means use Wikitionary).'),
-        'enable_negative_priority': _(
-            'If enabled, you will be able to set a negative priority to your '
-            'account in the Accounts window. BE CAREFUL, when you are logged '
-            'in with a negative priority, you will NOT receive any message '
-            'from your server.'),
         'file_transfers_port': '',
         'ft_add_hosts_to_send': _(
             'List of send hosts (comma separated) in '
@@ -668,13 +634,11 @@ ADVANCED_SETTINGS = {
             'If enabled, Gajim offers to download plugins hosted on gajim.org'),
         'providers_list_url': _(
             'Endpoint for retrieving a list of providers for sign up'),
+        'quick_reaction_emojis': _(
+            'Comma-separated list of emojis for quick reaction buttons'),
         'search_engine': '',
-        'send_on_ctrl_enter': _(
-            'Send message on Ctrl+Enter and make a new line with Enter.'),
         'stun_server': _('STUN server to use when using Jingle'),
         'time_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
-        'trayicon_notification_on_events': _(
-            'Notify of events in the notification area.'),
         'use_kib_mib': _(
             'IEC standard says KiB = 1024 bytes, KB = 1000 bytes.'),
         'use_stun_server': _(
