@@ -28,6 +28,9 @@ from gajim.common.storage.archive.const import MessageType
 from gajim.common.storage.archive.models import Message
 from gajim.common.types import ChatContactT
 from gajim.common.util.muc import message_needs_highlight
+from gajim.common.util.preview import GeoPreview
+from gajim.common.util.preview import get_preview_data
+from gajim.common.util.preview import UrlPreview
 from gajim.common.util.text import format_fingerprint
 from gajim.common.util.user_strings import get_moderation_text
 from gajim.common.util.user_strings import get_retraction_text
@@ -41,7 +44,8 @@ from gajim.gtk.conversation.rows.widgets import MessageIcons
 from gajim.gtk.conversation.rows.widgets import NicknameLabel
 from gajim.gtk.menus import GajimMenu
 from gajim.gtk.menus import get_chat_row_menu
-from gajim.gtk.preview import PreviewWidget
+from gajim.gtk.preview.geo import GeoPreviewWidget
+from gajim.gtk.preview.preview import PreviewWidget
 from gajim.gtk.referenced_message import ReferencedMessageNotFoundWidget
 from gajim.gtk.referenced_message import ReferencedMessageWidget
 from gajim.gtk.util.misc import container_remove_all
@@ -55,7 +59,6 @@ MERGE_TIMEFRAME = timedelta(seconds=120)
 
 class MessageRow(BaseRow):
     def __init__(self, contact: ChatContactT, message: Message) -> None:
-
         BaseRow.__init__(self, contact.account)
         self.set_selectable(True)
         self.type = "chat"
@@ -177,25 +180,29 @@ class MessageRow(BaseRow):
         assert message.text is not None
         self.text = message.text
 
-        if app.preview_manager.is_previewable(self.text, message.oob):
-            self._message_widget = PreviewWidget(self._contact.account)
-            app.preview_manager.create_preview(
-                self.text, self._message_widget, self._is_outgoing, self._muc_context
-            )
-        else:
-            if message.reply is not None:
-                referenced_message = message.get_referenced_message()
-                if referenced_message is None:
-                    self._ref_message_widget = ReferencedMessageNotFoundWidget()
-                else:
-                    self._ref_message_widget = ReferencedMessageWidget(
-                        self._contact, referenced_message
-                    )
+        match preview := get_preview_data(self.text, message.oob):
+            case UrlPreview():
+                self._message_widget = PreviewWidget(
+                    self._contact.account, preview, self._is_outgoing, self._muc_context
+                )
 
-            self._message_widget = MessageWidget(self._contact.account)
-            self._message_widget.add_with_styling(self.text, nickname=self.name)
-            if self._contact.is_groupchat and not self._is_outgoing:
-                self._apply_highlight(self.text)
+            case GeoPreview():
+                self._message_widget = GeoPreviewWidget(preview)
+
+            case _:
+                if message.reply is not None:
+                    referenced_message = message.get_referenced_message()
+                    if referenced_message is None:
+                        self._ref_message_widget = ReferencedMessageNotFoundWidget()
+                    else:
+                        self._ref_message_widget = ReferencedMessageWidget(
+                            self._contact, referenced_message
+                        )
+
+                self._message_widget = MessageWidget(self._contact.account)
+                self._message_widget.add_with_styling(self.text, nickname=self.name)
+                if self._contact.is_groupchat and not self._is_outgoing:
+                    self._apply_highlight(self.text)
 
         if self._ref_message_widget is not None:
             box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=3)
@@ -254,7 +261,6 @@ class MessageRow(BaseRow):
     def get_chat_row_menu(
         self,
     ) -> GajimMenu:
-
         copy_text = self._get_copy_text(self.get_text(), self.name, self.timestamp)
 
         return get_chat_row_menu(
@@ -291,7 +297,6 @@ class MessageRow(BaseRow):
     def _get_security_labels_data(
         self, security_labels: mod.SecurityLabel | None
     ) -> tuple[str, str] | None:
-
         if security_labels is None:
             return None
 
@@ -382,7 +387,6 @@ class MessageRow(BaseRow):
         self,
         encryption_data: mod.Encryption | None,
     ) -> tuple[str, str, str] | None:
-
         contact_encryption = self._contact.settings.get("encryption")
         if encryption_data is None:
             if not contact_encryption:
@@ -488,6 +492,8 @@ class MessageRow(BaseRow):
 
         if emoji in our_reactions:
             our_reactions.discard(emoji)
+        elif self._contact.reactions_per_user == 1:
+            our_reactions = {emoji}
         else:
             our_reactions.add(emoji)
 
