@@ -38,6 +38,7 @@ from nbxmpp.modules.vcard4 import OrgProperty
 from nbxmpp.modules.vcard4 import Parameters
 from nbxmpp.modules.vcard4 import PhotoProperty
 from nbxmpp.modules.vcard4 import ProdidProperty
+from nbxmpp.modules.vcard4 import PronounsProperty
 from nbxmpp.modules.vcard4 import RelatedProperty
 from nbxmpp.modules.vcard4 import RevProperty
 from nbxmpp.modules.vcard4 import RoleProperty
@@ -57,13 +58,13 @@ from gajim.common.iana import get_zone_data
 from gajim.common.iana import ZONE_DATA
 from gajim.common.util.text import escape_iri_path_segment
 from gajim.common.util.uri import InvalidUri
-from gajim.common.util.uri import open_uri
 from gajim.common.util.uri import parse_uri
 from gajim.common.util.uri import Uri
 
 from gajim.gtk.dropdown import GajimDropDown
 from gajim.gtk.util.classes import SignalManager
 from gajim.gtk.util.misc import convert_py_to_glib_datetime
+from gajim.gtk.util.misc import open_uri
 
 PropertyT = (
     AdrProperty
@@ -88,6 +89,7 @@ PropertyT = (
     | OrgProperty
     | PhotoProperty
     | ProdidProperty
+    | PronounsProperty
     | RelatedProperty
     | RevProperty
     | RoleProperty
@@ -112,6 +114,7 @@ SupportedPropertiesT = (
     | NoteProperty
     | NProperty
     | OrgProperty
+    | PronounsProperty
     | RoleProperty
     | TelProperty
     | TitleProperty
@@ -124,6 +127,7 @@ TextEntryPropertiesT = (
     | FnProperty
     | ImppProperty
     | OrgProperty
+    | PronounsProperty
     | RoleProperty
     | TelProperty
     | TitleProperty
@@ -135,6 +139,7 @@ LABEL_DICT = {
     "n": _("Name"),
     "bday": _("Birthday"),
     "gender": _("Gender"),
+    "pronouns": _("Pronouns"),
     "adr": p_("Profile", "Address"),
     "tel": _("Phone No."),
     "email": _("Email"),
@@ -170,6 +175,7 @@ DEFAULT_KWARGS: dict[str, dict[str, str | list[Any]]] = {
     "fn": {"value": ""},
     "bday": {"value": "", "value_type": "date"},
     "gender": {"sex": "", "identity": ""},
+    "pronouns": {"value": ""},
     "adr": {},
     "email": {"value": ""},
     "impp": {"value": ""},
@@ -196,6 +202,7 @@ PROPERTIES_WITH_TYPE = [
 ORDER = [
     "fn",
     "gender",
+    "pronouns",
     "bday",
     "adr",
     "email",
@@ -237,6 +244,7 @@ class VCardGrid(Gtk.Grid):
             "fn": TextEntryPropertyGui,
             "bday": DatePropertyGui,
             "gender": GenderPropertyGui,
+            "pronouns": TextEntryPropertyGui,
             "adr": AdrPropertyGui,
             "tel": TextEntryPropertyGui,
             "email": TextEntryPropertyGui,
@@ -308,6 +316,9 @@ class VCardGrid(Gtk.Grid):
         assert self._vcard is not None
         self._vcard.remove_property(prop.get_base_property())
 
+    def get_row_count(self) -> int:
+        return self._row_count
+
     def clear(self) -> None:
         self._vcard = None
         self._row_count = 0
@@ -500,7 +511,6 @@ class IdentityEntry(Gtk.Entry):
 
 
 class AdrBox(Gtk.Box, SignalManager):
-
     __gsignals__ = {
         "field-changed": (
             GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION,
@@ -1027,6 +1037,6 @@ class TzPropertyGui(VCardPropertyGui):
         item = dropdown.get_selected_item()
         assert item is not None
         assert isinstance(self._prop, TzProperty)
-        self._prop.value = item.props.key
+        self._prop.value = item.key
         self._value_label.update_text()
         self._time_label.update_text()
