@@ -1,5 +1,3 @@
-
-
 from .gajimplugin import GajimPlugin
 
-__all__ = ['GajimPlugin']
+__all__ = ["GajimPlugin"]
